/* 	=================================================================================

Demonstration of programming a microcontroller application using FSM style 

An example on how to implement a sequential system like a 1-digit BCD counter 
using the [Unit 2.3] style (present_state is an enumeration of states) 

TARGET MICROCONTROLLER: ATMEGA8535 (but the example is fully compatible with many other Atmel processors)

Target hardware and reference: Unit 4.9 http://digsys.upc.es/ed/CSD/units/Ch4/U4_09/Unit4_9.html#Project_3_Atmel

  	================================================================================= */

#include <avr/io.h>
//#include <stdlib.h>    
#include <avr/interrupt.h>


// Let's see where all the ATmega8535 microcontroller declarations are located.

// C:\Program Files (x86)\Atmel\Atmel Studio 6.2\devices\ATmega8535.xml
// #include "Atmel Toolchain/AVR8 GCC/Native/3.4.1056/avr8-gnu-toolchain/avr/include/avr/iom8535.h"


/* 	=================================================================================
	Definitions 
   	================================================================================= */

// ------------------ Inputs definitions ----------------------
// In case you like to read (poll) a single pin of a port, like the "CE_L", we'll use the masking technique:
#define CEnable_PORT	PINA			// The port where the input "CE_L" is located
#define CEnable_pin		0b00000010		// The pin number (PA1) where the CE_L" is located is the mask

// The CLK input, can be defined the same way, however, it won't be read but instead, the 
// interface of this input will be conducted through the INT0 interrupt 
#define CLK_PORT	PIND				// The port where the input CLK is located
#define CLK_pin		0b00000100			// The pin number where the input CLK is located (PD2/INT0)

// ------------------- Outputs definitions ----------------------
// Single pin outputs to be set to high or reset to low:
#define TC10_PORT			PORTA		// The port where the output "TC" is located 
#define TC10_pin_ON		0b00001000		// The pin number (PA6) where to perform a bitwise OR
#define TC10_pin_OFF	0b11110111		// The pin number (PA6) where to perform a bitwise AND


#define Q_out_PORT PORTC		// The port where the outputs of the counter are connected, 
								// and so it will be written


// Other definitions related to the counter system:

// The list of counter states 
#define 	Num_0 			'A'		//States enumeration. "Char"
#define 	Num_1 			'B'
#define 	Num_2 			'C'
#define 	Num_3 			'D'
#define 	Num_4 			'E'
#define 	Num_5 			'F'
#define 	Num_6 			'G'
#define 	Num_7 			'H'
#define 	Num_8 			'I'
#define 	Num_9 			'J'


/* 	=================================================================================
	Function prototypes
   	================================================================================= */

//#  define ISR(vector, [attributes]) Already defined in the header <avr/interrupt.h>
void init_system (void);
char state_logic(void);
char output_logic(void);

void read_inputs(void);
void write_outputs(void);


/* 	=================================================================================
	Global variables (RAM memory) to make the system compatible and as hardware independent as possible
  	================================================================================= */

char Terminal_Count;  // The variable to be written and easily watched while debugging.

char Q_out; 		// The output of the counter to be written to a port where the LED's
					//are connected

char CLK_Flag;		//The flag that will be set from the interrupt service routine
					// to indicate that a new falling edge has arrived

char Count_Enable;  //The variable to be watched easily which will be polled 


char present_state;		// The state variable which will be updated ("asynchronously")
						// every time that "state_logic()" is executed

/* 	=================================================================================
	Atmega8535 Configuration bit Fuses (datasheet page 238)
  	================================================================================= 
	  
	  This definitions are copied from the "iom8535.h" header file:
	  
	   #define FUSE_MEMORY_SIZE 2

	   // Low Fuse Byte 
	   #define FUSE_CKSEL0      (unsigned char)~_BV(0)	//CKSEL(3..0) = "1110" External resonator or crystal up to 8 MHz
	   #define FUSE_CKSEL1      (unsigned char)~_BV(1)
	   #define FUSE_CKSEL2      (unsigned char)~_BV(2)
	   #define FUSE_CKSEL3      (unsigned char)~_BV(3)
	   #define FUSE_SUT0        (unsigned char)~_BV(4)	//SUT(1..0) = "11" Crystal Oscillator, slowly rising power
	   #define FUSE_SUT1        (unsigned char)~_BV(5)
	   #define FUSE_BODEN       (unsigned char)~_BV(6)	// Brown out detector enable
	   #define FUSE_BODLEVEL    (unsigned char)~_BV(7)	// BODLEVEL = "0" ==> 4V
								// Brown-out Reset. The MCU is reset when the supply voltage VCC is below the
								// Brown-out Reset threshold (VBOT) and the Brown-out Detector is enabled.
	   
	   #define LFUSE_DEFAULT (FUSE_CKSEL1 & FUSE_CKSEL2 & FUSE_CKSEL3 & FUSE_SUT0 & FUSE_SUT1)
										// "0001" "00" Internal 1 MHz calibrated oscillator, 

	   // High Fuse Byte 
	   #define FUSE_BOOTRST     (unsigned char)~_BV(0)	//Select Reset Vector
	   #define FUSE_BOOTSZ0     (unsigned char)~_BV(1)	//The default value of BOOTSZ1..0 results in maximum Boot Size
	   #define FUSE_BOOTSZ1     (unsigned char)~_BV(2)	
	   #define FUSE_EESAVE      (unsigned char)~_BV(3)	//EEPROM memory is preserved through the Chip Erase
	   #define FUSE_CKOPT       (unsigned char)~_BV(4)	// When CKOPT is programmed, the Oscillator output will oscillate will a full rail-to-rail swing on the output.
	   #define FUSE_SPIEN       (unsigned char)~_BV(5)	// Enable Serial Programming
	   #define FUSE_WDTON       (unsigned char)~_BV(6)	// Enable the Watch Dog timer
	   #define FUSE_S8535C      (unsigned char)~_BV(7)	// Compatible to the AT90S8535
	   #define HFUSE_DEFAULT (FUSE_BOOTSZ0 & FUSE_BOOTSZ1 & FUSE_SPIEN)
	  */

// So, let's program the configuration bits for a given developing board and AVR chip:
// Do not change these values
    FUSES =
    {
		// let's use an external oscillator "1110", "01", and BODEN:
			    //.low = (FUSE_BODEN & FUSE_BODLEVEL),
		.low = (FUSE_CKSEL0 & FUSE_SUT1 & FUSE_BODEN & FUSE_BODLEVEL),
		.high = (FUSE_BOOTSZ0 & FUSE_BOOTSZ1 & FUSE_SPIEN & FUSE_EESAVE),
    }; 
	
	
/* 	=================================================================================
	Main function 
  	================================================================================= */
int main(void){
    init_system ();
    while(1) {                      /* loop forever the FSM */
            output_logic();
            state_logic();
    }
}	

/* 	=================================================================================
	Function definitions
  	================================================================================= */


//*****************************************************************************
//Initialisation operations (setup) 
//*****************************************************************************

void init_system (void){

// General microcontroller configuration bits

// Application program starts here:

	PORTA 	= 0x00;				// Let's not use the Pull-up resistor in the inputs 
								// (we'll wire external resistors like R4)
	DDRA 	= 0b00001000;		/* All inputs, except the PA3 that is the output TC10 */
	
	PORTD 	= 0x00;				// Let's not use the Pull-up resistor in PD2
								// Our CLK input will include R_CLK pull-up resistor

	DDRD 	= 0b00000000;		/* All inputs. The PD2 pin is our CLK (Using it for the interrupt INT0) 
									and the PD3 is the CE_L input */
								
	PORTC 	= 0x00;
	DDRC	= 0xF0; 			/* The pins PB7 ..PB4 of the PORTB, are outputs: the Q(3..0) */

	GICR = 1<<INT0;					// Enable INT0
	MCUCR = 1<<ISC01 | 0<<ISC00;	// Trigger INT0 on falling edge (interrupt after clicking the pushbutton)
		
	sei(); // Global interrupts allowed
	
	present_state = Num_0;
	
    CLK_Flag = 0;		// The flag to detect an INT0 interrupt

	// Set the initial output values
	Terminal_Count = 0;
	Q_out = 0;
	
	write_outputs();
}


//*****************************************************************************
//State variable logic routine 
//*****************************************************************************
char state_logic(void){
	char error = 0;
// This is a behavioural way to interpret and translate the truth table of the CC1 
	read_inputs ();
	
	if (CLK_Flag == 1){
		if (Count_Enable == 1){				// The system has to count
			switch (present_state) {
				case Num_0:
				present_state = Num_1;
				break;
				case Num_1:
				present_state = Num_2;
				break;
				case Num_2:
				present_state = Num_3;
				break;
				case Num_3:
				present_state = Num_4;
				break;
				case Num_4:
				present_state = Num_5;
				break;
				case Num_5:
				present_state = Num_6;
				break;
				case Num_6:
				present_state = Num_7;
				break;
				case Num_7:
				present_state = Num_8;
				break;
				case Num_8:
				present_state = Num_9;
				break;
				case Num_9:
				present_state = Num_0;
				break;

				default:
				error = 1;		// If no states have been selected, there is an error
				// and the systems must be stopped
				break;
			}
		}
		CLK_Flag = 0;	// Clear the CLK_flag, so it can be set again in the next falling edge in PD2
	}
	return (error);
}
	
	
//*****************************************************************************
//Output logic routine 
//*****************************************************************************
char output_logic(void){
	unsigned char error = 0;
	
// Calculate the outputs
	
	switch (present_state) {		// Set the outputs for every state:a truth table
		case Num_0:
			Q_out = 0;
		break;
		case Num_1:
			Q_out = 1;
		break;
		case Num_2:
			Q_out = 2;
		break;
		case Num_3:
			Q_out = 3;
		break;
		case Num_4:
			Q_out = 4;
		break;
		case Num_5:
			Q_out = 5;
		break;
		case Num_6:
			Q_out = 6;
		break;
		case Num_7:
			Q_out = 7;
		break;
		case Num_8:
			Q_out = 8;
		break;
		case Num_9:
			Q_out = 9;
		break;
		
		default:
			error = 1;		// If no states have been selected, there is an error
		// and the systems must be stopped
		break;
	}
	// Lets set the value of the Terminal_Count:
	if (present_state == Num_9 && Count_Enable == 1)
		Terminal_Count = 1;
	else
		Terminal_Count = 0;
	
	// Write the ports:
	write_outputs();
    
        /*  So, as you see, even if it takes more code, the process to code in C our applications will be:  
            1) Process or calculate the variables (to be able to watch them), and 
            2) Write the outputs to the corresponding pins.
			
			In this way, the program will be compatible for multiple microcontrollers and boards, 
			except for init_system() and read and write ports functions, whicll will behave like a kind of a driver
			for the specific hardware.
        */	
	return (error);
}


//***************************************************************************
//Interrupt Service Routine to attend the external INT0 (The PD2 pin)
//***************************************************************************

ISR(INT0_vect){
	cli();		// clear all sources of interrupt while attending this one

    CLK_Flag = 1;		
	/*	Let's set our convenient "software" flag, so we'll update the state every time that there is an interrupt 
		(the arrows (state transitions) of the state digram will be executed every time that there is a CLK falling edge) */
	                     
	sei();			//Enable general interrupts, so that the system can be interrupted again	
}

//***************************************************************************
// Write the output variables to the corresponding pins and ports 
//***************************************************************************

void write_outputs(void){

	__asm ("nop");	// Do nothing by now (no operate). This is a convenient way to start writing a function
		
	// Example of bitwise port control: a method of changing one or more bits of a parallel port without
	// affecting the other bits of the port.

	// This line demonstrates using the OR operator to force two bits of the port to '1'
	// without affecting the other bits of the port:
	//	PORTB = PORTB | 0b01100000;		// The bits 6 and 5 are set to '1'

	// This is an example on how to force a bit of the port to '0' by ANDing the bit with a 0 without
	// affecting the other bits of the port:
	// PORTB = PORTB & 0b11111110;		// The bit 0 is reset to '0'
	
	if (Terminal_Count == 1)
		TC10_PORT = TC10_PORT | TC10_pin_ON ;	// Sets the TC10 bit to '1'	(which means that the LED goes ON)
	else
		TC10_PORT = TC10_PORT & TC10_pin_OFF;	// Resets the TC10 bit to '0' (which means that the LED goes OFF)
	
	// Write the counter output port (which is active low):
	Q_out_PORT = Q_out<<4; // Shift 4-bits to the left and writes all the port pins
	
};
	
	
//***************************************************************************
// Write the output variables to the corresponding pins and ports 
//***************************************************************************

void read_inputs (void){

	__asm ("nop");	// do nothing by now

/* Polling --> Read the control input and set the "char" variable
// Let's read the CE_L input pin and set the convenient variable "Count_Enable" using the masking technique: */
	if ( CEnable_PORT & CEnable_pin )	
		Count_Enable = 0;
	else
		Count_Enable = 1;

};
		